@@ -8,8 +8,18 @@
 #ifndef MIN
 #define MIN(a, b)        ((a) < (b) ? (a) : (b))
 #endif
+#ifndef M_PI
+#define M_PI           3.14159265358979323846  /* pi */
+#endif
+#ifndef M_PI_4
+#define M_PI_4         0.78539816339744830962  /* pi/4 */
+#endif
 #define boxgap 25
 
+#ifdef LIBUNDO
+#include "undo.h"
+#endif
+
 void
 del_struct (struct data *hpc)
 /* remove the data structure corresponding to a bond */
@@ -52,7 +62,7 @@ select_char (int mx, int my, float s_f)
 }
 
 void
-add_char (int x, int y, char *c, int direct, int marked)
+add_char (int x, int y, char *c, int direct, int marked, int color, int font, int size)
 /* add a new label */
 {
   struct dc *last;
@@ -62,6 +72,9 @@ add_char (int x, int y, char *c, int direct, int marked)
   new_c->y = y;
   new_c->direct = direct;
   new_c->marked = marked;
+  new_c->color = color;
+  new_c->font = font;
+  new_c->size = size;
 #ifdef LIBUNDO
   new_c->next = (struct dc *) undo_malloc (sizeof (struct dc));
 #else
@@ -104,15 +117,16 @@ round_coord (int i, float step)
 }
 
 void
-add_struct (int x, int y, int tx, int ty, int bond, int smarked, int tmarked, int decoration)
+add_struct (int x, int y, int tx, int ty, int bond, int smarked, int tmarked, int decoration, int color)
 /* adds the data corresponding to a new bond */
 {
   struct data *last;
 
   if (bond == 17)
     {
-      add_spline (x, y, x + 0.5 * (tx - x), y - 0.5 * (ty - y),
-		  x + 0.5 * (tx - x), y - 0.5 * (ty - y), tx, ty, 1, smarked);
+      add_spline (x, y, (int) (x + 0.5 * (tx - x)), (int)(y - 0.5 * (ty - y)),
+		 (int) (x + 0.5 * (tx - x)), (int)(y - 0.5 * (ty - y)), 
+		 tx, ty, 1, smarked,color);
       return;
     }
 
@@ -124,6 +138,7 @@ add_struct (int x, int y, int tx, int ty, int bond, int smarked, int tmarked, in
   new->smarked = smarked;
   new->tmarked = tmarked;
   new->decoration = decoration;
+  new->color = color;
 #ifdef LIBUNDO
   new->next = (struct data *) undo_malloc (sizeof (struct data));
 #else
@@ -138,7 +153,7 @@ add_struct (int x, int y, int tx, int ty, int bond, int smarked, int tmarked, in
 
 void
 add_spline (int x0, int y0, int x1, int y1, int x2, int y2, int x3, int y3,
-	    int type, int marked)
+	    int type, int marked, int color)
 /* adds the data corresponding to a new spline curve*/
 {
   struct spline *last;
@@ -153,6 +168,7 @@ add_spline (int x0, int y0, int x1, int y1, int x2, int y2, int x3, int y3,
   sp_new->y3 = y3;
   sp_new->type = type;
   sp_new->marked = marked;
+  sp_new->color = color;
 #ifdef LIBUNDO
   sp_new->next = (struct spline *) undo_malloc (sizeof (struct spline));
 #else
@@ -167,7 +183,7 @@ add_spline (int x0, int y0, int x1, int y1, int x2, int y2, int x3, int y3,
 
 void
 del_spline (struct spline *hpsp)
-/* remove the data structure corresponding to a bond */
+/* remove the data structure corresponding to a spline curve */
 {
   if (hp->nsp && hpsp)
     {
@@ -226,50 +242,61 @@ multi_bonds (int mx, int my, int mtx, int mty, int r)
 
   dx = mtx - mx;
   dy = mty - my;
-  v = calc_vector (dx, dy);
+  v = calc_vector (abs(dx), abs(dy));
   alpha = (float) asin ((float) dx / v);
   if ((dx < 0 && dy < 0) || !dy)
     {
-      x = r * cos (alpha - M_PI_4);
-      y = r * sin (alpha - M_PI_4);
-      tx = r * cos (alpha + M_PI_4);
-      ty = r * sin (alpha + M_PI_4);
+      x = (int) (r * cos (alpha - M_PI_4));
+      y = (int) (r * sin (alpha - M_PI_4));
+      tx = (int) (r * cos (alpha + M_PI_4));
+      ty = (int) (r * sin (alpha + M_PI_4));
     }
   if (dx > 0 && dy > 0)
     {
-      x = -r * cos (alpha + M_PI_4);
-      y = r * sin (alpha + M_PI_4);
-      tx = -r * cos (alpha - M_PI_4);
-      ty = r * sin (alpha - M_PI_4);
+      x = (int)(-r * cos (alpha + M_PI_4));
+      y = (int)(r * sin (alpha + M_PI_4));
+      tx = (int)(-r * cos (alpha - M_PI_4));
+      ty = (int)(r * sin (alpha - M_PI_4));
     }
   if (dx < 0 && dy > 0)
     {
-      x = -r * cos (M_PI + alpha - M_PI_4);
-      y = r * sin (M_PI + alpha - M_PI_4);
-      tx = r * cos (alpha + M_PI_4);
-      ty = -r * sin (alpha + M_PI_4);
+      x = (int) (-r * cos (M_PI + alpha - M_PI_4));
+      y = (int) (r * sin (M_PI + alpha - M_PI_4));
+      tx = (int) (r * cos (alpha + M_PI_4));
+      ty = (int) (-r * sin (alpha + M_PI_4));
     }
   if (dx > 0 && dy < 0)
     {
-      x = -r * cos (alpha + M_PI_4);
-      y = -r * sin (alpha + M_PI_4);
-      tx = -r * cos (alpha - M_PI_4);
-      ty = -r * sin (alpha - M_PI_4);
+      x = (int) (-r * cos (alpha + M_PI_4));
+      y = (int) (-r * sin (alpha + M_PI_4));
+      tx = (int) (-r * cos (alpha - M_PI_4));
+      ty = (int) ( -r * sin (alpha - M_PI_4));
     }
   if (!dx && dy < 0)
     {
-      x = -r * cos (M_PI_4);
-      y = -r * sin (M_PI_4);
-      tx = -r * cos (M_PI_4);
-      ty = r * sin (M_PI_4);
+      x = (int) (-r * cos (M_PI_4));
+      y = (int) (-r * sin (M_PI_4));
+      tx = (int) (-r * cos (M_PI_4));
+      ty = (int) (r * sin (M_PI_4));
     }
   if (!dx && dy > 0)
     {
-      x = r * cos (M_PI_4);
-      y = r * sin (M_PI_4);
-      tx = r * cos (M_PI_4);
-      ty = -r * sin (M_PI_4);
+      x = (int) (r * cos (M_PI_4));
+      y = (int) (r * sin (M_PI_4));
+      tx = (int) (r * cos (M_PI_4));
+      ty = (int) (-r * sin (M_PI_4));
     }
+
+      x = (int) (-r * cos (alpha + M_PI_4));
+      y = (int) (-r * sin (alpha + M_PI_4));
+      tx = (int)( -r * cos (alpha - M_PI_4));
+      ty = (int) (-r * sin (alpha - M_PI_4));
+if (dy >0) {
+      x = (int) (r * cos (alpha - M_PI_4));
+      y = (int) (-r * sin (alpha - M_PI_4));
+      tx = (int) (r * cos (alpha + M_PI_4));
+      ty = (int) (-r * sin (alpha + M_PI_4));
+}
   coord->x = mx + x;
   coord->y = my + y;
   coord->tx = mtx + tx;
@@ -295,69 +322,69 @@ center_double_bond (int mx, int my, int mtx, int mty, int r)
 
   if ((dx < 0 && dy < 0) || !dy)
     {
-      x = r * cos (alpha - M_PI);
-      y = r * sin (alpha - M_PI);
-      tx = r * cos (alpha + M_PI);
-      ty = r * sin (alpha + M_PI);
-      sx = -r * cos (alpha - M_PI);
-      sy = -r * sin (alpha - M_PI);
-      stx = -r * cos (alpha + M_PI);
-      sty = -r * sin (alpha + M_PI);
+      x = (int) (r * cos (alpha - M_PI));
+      y = (int) (r * sin (alpha - M_PI));
+      tx = (int) (r * cos (alpha + M_PI));
+      ty = (int) (r * sin (alpha + M_PI));
+      sx = (int) (-r * cos (alpha - M_PI));
+      sy = (int) (-r * sin (alpha - M_PI));
+      stx = (int) (-r * cos (alpha + M_PI));
+      sty = (int) (-r * sin (alpha + M_PI));
     }
   if (dx > 0 && dy > 0)
     {
-      x = -r * cos (alpha + M_PI);
-      y = r * sin (alpha + M_PI);
-      tx = -r * cos (alpha - M_PI);
-      ty = r * sin (alpha - M_PI);
-      sx = r * cos (alpha + M_PI);
-      sy = -r * sin (alpha + M_PI);
-      stx = r * cos (alpha - M_PI);
-      sty = -r * sin (alpha - M_PI);
+      x = (int) (-r * cos (alpha + M_PI));
+      y = (int) (r * sin (alpha + M_PI));
+      tx = (int) (-r * cos (alpha - M_PI));
+      ty = (int) (r * sin (alpha - M_PI));
+      sx = (int) (r * cos (alpha + M_PI));
+      sy = (int) (-r * sin (alpha + M_PI));
+      stx = (int) (r * cos (alpha - M_PI));
+      sty = (int) (-r * sin (alpha - M_PI));
     }
   if (dx < 0 && dy > 0)
     {
-      x = -r * cos (M_PI + alpha - M_PI);
-      y = r * sin (M_PI + alpha - M_PI);
-      tx = r * cos (alpha + M_PI);
-      ty = -r * sin (alpha + M_PI);
-      sx = r * cos (M_PI + alpha - M_PI);
-      sy = -r * sin (M_PI + alpha - M_PI);
-      stx = -r * cos (alpha + M_PI);
-      sty = r * sin (alpha + M_PI);
+      x = (int) (-r * cos (M_PI + alpha - M_PI));
+      y = (int) (r * sin (M_PI + alpha - M_PI));
+      tx = (int) (r * cos (alpha + M_PI));
+      ty = (int) (-r * sin (alpha + M_PI));
+      sx = (int) (r * cos (M_PI + alpha - M_PI));
+      sy = (int) (-r * sin (M_PI + alpha - M_PI));
+      stx = (int) (-r * cos (alpha + M_PI));
+      sty = (int) (r * sin (alpha + M_PI));
     }
   if (dx > 0 && dy < 0)
     {
-      x = -r * cos (alpha + M_PI);
-      y = -r * sin (alpha + M_PI);
-      tx = -r * cos (alpha - M_PI);
-      ty = -r * sin (alpha - M_PI);
-      sx = r * cos (alpha + M_PI);
-      sy = r * sin (alpha + M_PI);
-      stx = r * cos (alpha - M_PI);
-      sty = r * sin (alpha - M_PI);
+      x = (int) (-r * cos (alpha + M_PI));
+      y = (int) (-r * sin (alpha + M_PI));
+      tx = (int) (-r * cos (alpha - M_PI));
+      ty = (int) (-r * sin (alpha - M_PI));
+      sx = (int) (r * cos (alpha + M_PI));
+      sy = (int) (r * sin (alpha + M_PI));
+      stx = (int) (r * cos (alpha - M_PI));
+      sty = (int) (r * sin (alpha - M_PI));
     }
   if (!dx && dy < 0)
     {
-      x = -r * cos (M_PI);
-      y = -r * sin (M_PI);
-      tx = -r * cos (M_PI);
-      ty = r * sin (M_PI);
-      sx = r * cos (M_PI);
-      sy = r * sin (M_PI);
-      stx = r * cos (M_PI);
-      sty = -r * sin (M_PI);
+      x =  (int) (-r * cos (M_PI));
+      y =  (int) (-r * sin (M_PI));
+      tx =  (int) (-r * cos (M_PI));
+      ty =  (int) (r * sin (M_PI));
+      sx =  (int) (r * cos (M_PI));
+      sy =  (int) (r * sin (M_PI));
+      stx =  (int) (r * cos (M_PI));
+      sty =  (int) (-r * sin (M_PI));
     }
   if (!dx && dy > 0)
     {
-      x = r * cos (M_PI);
-      y = r * sin (M_PI);
-      tx = r * cos (M_PI);
-      ty = -r * sin (M_PI);
-      sx = -r * cos (M_PI);
-      sy = -r * sin (M_PI);
-      stx = -r * cos (M_PI);
-      sty = r * sin (M_PI);
+      x =  (int) (r * cos (M_PI));
+      y =  (int) (r * sin (M_PI));
+      tx =  (int) (r * cos (M_PI));
+      ty =  (int) (-r * sin (M_PI));
+      sx =  (int) (-r * cos (M_PI));
+      sy =  (int) (-r * sin (M_PI));
+      stx =  (int) (-r * cos (M_PI));
+      sty =  (int) (r * sin (M_PI));
     }
 
   coord->x = mx + x;
@@ -375,7 +402,7 @@ int
 calc_vector (int dx, int dy)
 /* returns the length of a bond vector */
 {
-  return ((int) sqrt (dx * dx + dy * dy));
+  return ((int) sqrt ((double)(dx * dx + dy * dy)));
 }
 
 struct xy_co *
@@ -546,6 +573,30 @@ position (int x, int y, int ax, int ay, int ang_type)
 	      bx = hx;
 	      by = hy;
 	    }
+/*retry at double distance */
+	  hx = 2*hx-ax ;
+	  hy = 2*hy-ay ;
+	  dx = abs (x - hx);
+	  dy = abs (y - hy);
+	  v = calc_vector (dx, dy);
+	  if (v < bv)
+	    {
+	      bv = v;
+	      bx = hx;
+	      by = hy;
+	    }
+/*retry at greater distance */
+	  hx = 3*coord->x-2*ax ;
+	  hy = 3*coord->y-2*ay ;
+	  dx = abs (x - hx);
+	  dy = abs (y - hy);
+	  v = calc_vector (dx, dy);
+	  if (v < bv)
+	    {
+	      bv = v;
+	      bx = hx;
+	      by = hy;
+	    }
 	}
     }
   coord->x = bx;
@@ -553,6 +604,23 @@ position (int x, int y, int ax, int ay, int ang_type)
   return (coord);
 }
 
+struct xy_co *
+intersect (int ax, int ay, int atx, int aty, int bx, int by, int btx, int bty)
+/* returns the intersection of two lines */
+{
+struct xy_co *coord = &xycoord[0];
+  /* the direction vectors of the two lines */
+  int avx = atx - ax, avy = aty - ay, bvx = btx - bx, bvy = bty - by; 
+  float mu; /* the factor of the second line */
+
+  mu = (float)(avx * (ay - by) + avy * (bx - ax)) / (float)(bvy * avx - avy * bvx);
+
+  coord->x = bx + (int)rint(mu * bvx);
+  coord->y = by + (int)rint(mu * bvy);
+
+  return (coord);
+}
+
 void
 setup_data (void)
 /* initializes the bond and label arrays */
@@ -692,6 +760,12 @@ partial_delete (int x, int y, int tx, int ty)
       if (!hp_spline)
 	break;
     }
+    if (addflag ==1) 
+    	if (refmark==1){
+    		addflag=0;
+    		refmark=0;
+    		refx=refy=0;
+    		}
   if (r)
     {
       FreePix ();
@@ -709,8 +783,13 @@ get_center_marked (int *x, int *y)
   struct dc *hp_atom;
   struct spline *hp_spline;
   int d;
-  int xmax = 0, ymax = 0;
-  int xmin = head.width, ymin = head.height;
+  int xmax,ymax,xmin,ymin;
+  
+  xmin=1000000;
+  ymin=1000000;
+  xmax=-1000000;
+  ymax=-1000000;
+  
   if (hp->n || hp->nc || hp->nsp)
     {
       hp_atom = dac_root.next;
@@ -794,9 +873,9 @@ get_center_marked (int *x, int *y)
 	    }
 	  hp_spline = hp_spline->next;
 	}
+    }
       *x = (xmin + xmax) / 2;
       *y = (ymin + ymax) / 2;
-    }
   return;
 }
 
@@ -945,8 +1024,8 @@ cut_end (int *x, int *y, int tx, int ty, int r)
   dy = ty - *y;
   v = calc_vector (dx, dy);
   alpha = (float) asin ((float) dx / v);
-  qx = r * sin (alpha);
-  qy = r * cos (alpha);
+  qx = (int) (r * sin (alpha));
+  qy = (int) (r * cos (alpha));
 
   if (dx < 0 && dy < 0)
     {
@@ -1006,7 +1085,7 @@ bond_cut (int x, int y, int tx, int ty, int r)
   for (d = 0; d < hp->nc; d++)
     {
       hc = hp_a->c;
-      n = strlen (hp_a->c);
+      n = (int)strlen (hp_a->c);
       for (i = 0; i < n; i++)
 	{
 	  if (Extra_char (*hc))
@@ -1122,7 +1201,16 @@ move_pos (int x, int y, int tx, int ty)
       hp_spline = hp_spline->next;
     }
 
-
+	if ( !f) {
+	hp_bond = select_vector(x,y, 1.);
+	if (hp_bond) {
+	      hp_bond->x += tx-x;
+	      hp_bond->y += ty-y;
+	      hp_bond->tx += tx-x;
+	      hp_bond->ty += ty-y;
+		f = True;
+		}
+	}
   return f;
 }
 
@@ -1187,11 +1275,16 @@ partial_move (int x, int y, int tx, int ty, int dx, int dy)
 	    break;
 	}
     }
+    if (addflag==1 && refmark ==1)
+    			 {
+    			refx+=dx;
+    			refy+=dy;
+    			}
   return f;
 }
 
 int
-partial_rotate (int xcent, int ycent, double rotsin, double rotcos)
+partial_rotate (/*int xcent, int ycent,*/ double rotsin, double rotcos)
 /* rotates all marked bonds and labels by a given angle */
 {
   struct data *hp_bond;
@@ -1209,8 +1302,8 @@ partial_rotate (int xcent, int ycent, double rotsin, double rotcos)
     {
       if (hp_bond->smarked == 1)
 	{
-	  rx = tmpx[dtmp];
-	  ry = tmpy[dtmp++];
+	  rx = (double)tmpx[dtmp];
+	  ry = (double)tmpy[dtmp++];
 	  tmp = rx * rotcos - ry * rotsin;
 	  ry = rx * rotsin + ry * rotcos;
 	  tmp = tmp + copysign (0.5, tmp);
@@ -1221,8 +1314,8 @@ partial_rotate (int xcent, int ycent, double rotsin, double rotcos)
 	}
       if (hp_bond->tmarked == 1)
 	{
-	  rx = tmpx[dtmp];
-	  ry = tmpy[dtmp++];
+	  rx = (double)tmpx[dtmp];
+	  ry = (double)tmpy[dtmp++];
 	  tmp = rx * rotcos - ry * rotsin;
 	  tmp = tmp + copysign (0.5, tmp);
 	  ry = rx * rotsin + ry * rotcos;
@@ -1239,8 +1332,8 @@ partial_rotate (int xcent, int ycent, double rotsin, double rotcos)
     {
       if (hp_atom->marked == 1)
 	{
-	  rx = tmpx[dtmp];
-	  ry = tmpy[dtmp++];
+	  rx = (double)tmpx[dtmp];
+	  ry = (double)tmpy[dtmp++];
 	  tmp = rx * rotcos - ry * rotsin;
 	  tmp = tmp + copysign (0.5, tmp);
 	  ry = rx * rotsin + ry * rotcos;
@@ -1257,32 +1350,32 @@ partial_rotate (int xcent, int ycent, double rotsin, double rotcos)
     {
       if (hp_spline->marked == 1)
 	{
-	  rx = tmpx[dtmp];
-	  ry = tmpy[dtmp++];
+	  rx = (double)tmpx[dtmp];
+	  ry = (double)tmpy[dtmp++];
 	  tmp = rx * rotcos - ry * rotsin;
 	  tmp = tmp + copysign (0.5, tmp);
 	  ry = rx * rotsin + ry * rotcos;
 	  ry = ry + copysign (0.5, ry);
 	  hp_spline->x0 = (int) tmp + xcent;
 	  hp_spline->y0 = (int) ry + ycent;
-	  rx = tmpx[dtmp];
-	  ry = tmpy[dtmp++];
+	  rx = (double)tmpx[dtmp];
+	  ry = (double)tmpy[dtmp++];
 	  tmp = rx * rotcos - ry * rotsin;
 	  tmp = tmp + copysign (0.5, tmp);
 	  ry = rx * rotsin + ry * rotcos;
 	  ry = ry + copysign (0.5, ry);
 	  hp_spline->x1 = (int) tmp + xcent;
 	  hp_spline->y1 = (int) ry + ycent;
-	  rx = tmpx[dtmp];
-	  ry = tmpy[dtmp++];
+	  rx = (double)tmpx[dtmp];
+	  ry = (double)tmpy[dtmp++];
 	  tmp = rx * rotcos - ry * rotsin;
 	  tmp = tmp + copysign (0.5, tmp);
 	  ry = rx * rotsin + ry * rotcos;
 	  ry = ry + copysign (0.5, ry);
 	  hp_spline->x2 = (int) tmp + xcent;
 	  hp_spline->y2 = (int) ry + ycent;
-	  rx = tmpx[dtmp];
-	  ry = tmpy[dtmp++];
+	  rx = (double)tmpx[dtmp];
+	  ry = (double)tmpy[dtmp++];
 	  tmp = rx * rotcos - ry * rotsin;
 	  tmp = tmp + copysign (0.5, tmp);
 	  ry = rx * rotsin + ry * rotcos;
@@ -1293,6 +1386,16 @@ partial_rotate (int xcent, int ycent, double rotsin, double rotcos)
 	}
       hp_spline = hp_spline->next;
     }
+	if (addflag==1 && refmark==1){
+	  rx = (double)tmpx[dtmp];
+	  ry = (double)tmpy[dtmp++];
+	  tmp = rx * rotcos - ry * rotsin;
+	  tmp = tmp + copysign (0.5, tmp);
+	  ry = rx * rotsin + ry * rotcos;
+	  ry = ry + copysign (0.5, ry);
+	  refx = (int) tmp + xcent;
+	  refy = (int) ry + ycent;
+	}
   return f;
 }
 
@@ -1319,6 +1422,23 @@ flip_horiz ()
 	{
 	  hp_bond->x = hp_bond->x - 2 * (hp_bond->x - xref);
 	  hp_bond->tx = hp_bond->tx - 2 * (hp_bond->tx - xref);
+	  switch (hp_bond->bond)
+	  {
+	  	case 1:
+	  		hp_bond->bond=2;
+	  		break;
+	  	case 2: 
+	  		hp_bond->bond=1;
+	  		break;
+	  	case 14:
+	  		hp_bond->bond=15;
+	  		break;
+	  	case 15:
+	  		hp_bond->bond=14;
+	  		break;
+	  	default:
+	  		break;
+	  	}					 
 	}
       hp_bond = hp_bond->next;
     }
@@ -1346,6 +1466,9 @@ flip_horiz ()
 	    break;
 	}
     }
+    if (addflag==1 && refmark ==1 ) 
+    		refx = refx - 2* (refx-xref);
+	
   FreePix ();
   CreatePix ();
   Display_Mol ();
@@ -1375,6 +1498,23 @@ flip_vert ()
 	{
 	  hp_bond->y = hp_bond->y - 2 * (hp_bond->y - yref);
 	  hp_bond->ty = hp_bond->ty - 2 * (hp_bond->ty - yref);
+	  switch (hp_bond->bond)
+	  {
+	  	case 1:
+	  		hp_bond->bond=2;
+	  		break;
+	  	case 2: 
+	  		hp_bond->bond=1;
+	  		break;
+	  	case 14:
+	  		hp_bond->bond=15;
+	  		break;
+	  	case 15:
+	  		hp_bond->bond=14;
+	  		break;
+	  	default:
+	  		break;
+	  	}					 
 	}
       hp_bond = hp_bond->next;
     }
@@ -1402,6 +1542,9 @@ flip_vert ()
 	}
     }
 
+    if (addflag==1 && refmark ==1 )
+    		refy = refy - 2* (refy-yref);
+
   FreePix ();
   CreatePix ();
   Display_Mol ();
@@ -1431,7 +1574,7 @@ linked to something outside the box, are ignored. **/
     {
       if (hp_bond->smarked && hp_bond->tmarked)
 	add_struct (hp_bond->x + 5, hp_bond->y + 5,
-		    hp_bond->tx + 5, hp_bond->ty + 5, hp_bond->bond, 1, 1,hp_bond->decoration);
+		    hp_bond->tx + 5, hp_bond->ty + 5, hp_bond->bond, 1, 1,hp_bond->decoration, hp_bond->color);
       hp_bond->smarked = hp_bond->tmarked = 0;
       hp_bond = hp_bond->next;
     }
@@ -1439,7 +1582,7 @@ linked to something outside the box, are ignored. **/
     {
       if (hp_atom->marked)
 	add_char (hp_atom->x + 5, hp_atom->y + 5, hp_atom->c,
-		  hp_atom->direct, 1);
+		  hp_atom->direct, 1, hp_atom->color,hp_atom->font,hp_atom->size);
       hp_atom->marked = 0;
       hp_atom = hp_atom->next;
     }
@@ -1453,7 +1596,7 @@ linked to something outside the box, are ignored. **/
 		      hp_spline->x1 + 5, hp_spline->y1 + 5,
 		      hp_spline->x2 + 5, hp_spline->y2 + 5,
 		      hp_spline->x3 + 5, hp_spline->y3 + 5,
-		      hp_spline->type, 1);
+		      hp_spline->type, 1, hp_spline->color);
 	  hp_spline->marked = 0;
 	}
       hp_spline = hp_spline->next;
@@ -1481,7 +1624,7 @@ partial_rescale (int x, int y, int tx, int ty, int updown, int aniso)
     factor = 1. - abs (updown) / 10.;
   else
     factor = 1. + updown / 10.;
-
+  if (factor < 0.) return f; /* negative scale would cause strange inversions */
   if (aniso)
     factorx = 1.;
   else
@@ -1520,6 +1663,14 @@ partial_rescale (int x, int y, int tx, int ty, int updown, int aniso)
 	  ry = tmpy[dtmp++];
 	  hp_atom->x = (int) (rx * factorx) + xcent;
 	  hp_atom->y = (int) (ry * factor) + ycent;
+	  if (factor < 0.9) hp_atom->size -=1 ;
+	  if (factor < 0.6) hp_atom->size -=1 ;
+	  if (factor < 0.3) hp_atom->size -=1 ;
+	  if (factor > 1.1) hp_atom->size +=1 ;
+	  if (factor > 1.3) hp_atom->size +=1 ;
+	  if (factor > 1.6) hp_atom->size +=1 ;
+	  if (hp_atom->size <0) hp_atom->size=0;
+	  if (hp_atom->size >6) hp_atom->size=6;
 	  f = True;
 	}
       hp_atom = hp_atom->next;
@@ -1550,6 +1701,13 @@ partial_rescale (int x, int y, int tx, int ty, int updown, int aniso)
 	}
       hp_spline = hp_spline->next;
     }
+	if (addflag == 1 && refmark == 1) {
+	  rx = tmpx[dtmp];
+	  ry = tmpy[dtmp++];
+	  refx = (int) (rx * factorx) + xcent;
+	  refy = (int) (ry * factor) + ycent;
+	  }
+	  
   return f;
 }
 
@@ -1569,32 +1727,36 @@ tidy_mol ()
   hn = hp->n;
   for (d = 0; d < hn; d++)	/* for all bonds */
     {
-	if (hp_bond->bond == 11) continue;
+	if (hp_bond->bond == 11) continue; /* do not bother with circles */
+	/* straigthen near-vertical or near-horizontal bonds */
+	if (abs (hp_bond->y - hp_bond->ty) ==1) hp_bond->ty=hp_bond->y;
+	if (abs (hp_bond->x - hp_bond->tx) ==1) hp_bond->tx=hp_bond->x;
+	 
       tmp_bond = da_root.next;
       for (t = 0; t < d; t++)	/* check against all preceding bonds */
 	{
 	if (tmp_bond->bond == 11) continue;
 /* if startpoint or endpoint very close to another, snap to that */
-	  if (fabs (hp_bond->x - tmp_bond->x) < 5 &&
-	      fabs (hp_bond->y - tmp_bond->y) < 5)
+	  if (abs (hp_bond->x - tmp_bond->x) < 5 &&
+	      abs (hp_bond->y - tmp_bond->y) < 5)
 	    {
 	      hp_bond->x = tmp_bond->x;
 	      hp_bond->y = tmp_bond->y;
 	    }
-	  if (fabs (hp_bond->tx - tmp_bond->x) < 5 &&
-	      fabs (hp_bond->ty - tmp_bond->y) < 5)
+	  if (abs (hp_bond->tx - tmp_bond->x) < 5 &&
+	      abs (hp_bond->ty - tmp_bond->y) < 5)
 	    {
 	      hp_bond->tx = tmp_bond->x;
 	      hp_bond->ty = tmp_bond->y;
 	    }
-	  if (fabs (hp_bond->tx - tmp_bond->tx) < 5 &&
-	      fabs (hp_bond->ty - tmp_bond->ty) < 5)
+	  if (abs (hp_bond->tx - tmp_bond->tx) < 5 &&
+	      abs (hp_bond->ty - tmp_bond->ty) < 5)
 	    {
 	      hp_bond->tx = tmp_bond->tx;
 	      hp_bond->ty = tmp_bond->ty;
 	    }
-	  if (fabs (hp_bond->x - tmp_bond->tx) < 5 &&
-	      fabs (hp_bond->y - tmp_bond->ty) < 5)
+	  if (abs (hp_bond->x - tmp_bond->tx) < 5 &&
+	      abs (hp_bond->y - tmp_bond->ty) < 5)
 	    {
 	      hp_bond->x = tmp_bond->tx;
 	      hp_bond->y = tmp_bond->ty;
@@ -1603,20 +1765,20 @@ tidy_mol ()
 
 /* if startpoint _and_ endpoint overlap another bond, mark for deletion */
 
-	  if (fabs (hp_bond->x - tmp_bond->x) < 5 &&
-	      fabs (hp_bond->y - tmp_bond->y) < 5 &&
-	      fabs (hp_bond->tx - tmp_bond->tx) < 5 &&
-	      fabs (hp_bond->ty - tmp_bond->ty) < 5)
+	  if (abs (hp_bond->x - tmp_bond->x) < 5 &&
+	      abs (hp_bond->y - tmp_bond->y) < 5 &&
+	      abs (hp_bond->tx - tmp_bond->tx) < 5 &&
+	      abs (hp_bond->ty - tmp_bond->ty) < 5)
 	    {
 	      if (hp_bond->bond <= tmp_bond->bond)
 		hp_bond->smarked = hp_bond->tmarked = 1;
 	      else
 		tmp_bond->smarked = tmp_bond->tmarked = 1;
 	    }
-	  if (fabs (hp_bond->tx - tmp_bond->x) < 5 &&
-	      fabs (hp_bond->ty - tmp_bond->y) < 5 &&
-	      fabs (hp_bond->x - tmp_bond->tx) < 5 &&
-	      fabs (hp_bond->y - tmp_bond->ty) < 5)
+	  if (abs (hp_bond->tx - tmp_bond->x) < 5 &&
+	      abs (hp_bond->ty - tmp_bond->y) < 5 &&
+	      abs (hp_bond->x - tmp_bond->tx) < 5 &&
+	      abs (hp_bond->y - tmp_bond->ty) < 5)
 	    {
 	      if (hp_bond->bond <= tmp_bond->bond)
 		hp_bond->smarked = hp_bond->tmarked = 1;
@@ -1631,14 +1793,14 @@ tidy_mol ()
       for (t = 0; t < hnc; t++)
 	{
 /* if a label is close to either startpoint or endpoint, move it */
-	  if (fabs (hp_atom->x - hp_bond->x) < 5 &&
-	      fabs (hp_atom->y - hp_bond->y) < 5)
+	  if (abs (hp_atom->x - hp_bond->x) < 5 &&
+	      abs (hp_atom->y - hp_bond->y) < 5)
 	    {
 	      hp_atom->x = hp_bond->x;
 	      hp_atom->y = hp_bond->y;
 	    }
-	  if (fabs (hp_atom->x - hp_bond->tx) < 5 &&
-	      fabs (hp_atom->y - hp_bond->ty) < 5)
+	  if (abs (hp_atom->x - hp_bond->tx) < 5 &&
+	      abs (hp_atom->y - hp_bond->ty) < 5)
 	    {
 	      hp_atom->x = hp_bond->tx;
 	      hp_atom->y = hp_bond->ty;
@@ -1693,16 +1855,16 @@ add_bracket ()
   if (importflag != 0)
     return;
   gauge = 10 + (boxgap * mark.h / 600 * mark.w / 600);
-  add_struct (mark.x, mark.y, mark.x + gauge, mark.y, 0, 0, 0, 1);
-  add_struct (mark.x, mark.y, mark.x, mark.y + mark.h, 0, 0, 0, 1);
+  add_struct (mark.x, mark.y, mark.x + gauge, mark.y, 0, 0, 0, 1, curpen);
+  add_struct (mark.x, mark.y, mark.x, mark.y + mark.h, 0, 0, 0, 1, curpen);
   add_struct (mark.x, mark.y + mark.h, mark.x + gauge, mark.y + mark.h, 0, 0,
-	      0, 1);
+	      0, 1, curpen);
   add_struct (mark.x + mark.w - gauge, mark.y, mark.x + mark.w, mark.y, 0, 0,
-	      0, 1);
+	      0, 1, curpen);
   add_struct (mark.x + mark.w, mark.y, mark.x + mark.w, mark.y + mark.h, 0, 0,
-	      0, 1);
+	      0, 1, curpen);
   add_struct (mark.x + mark.w - gauge, mark.y + mark.h, mark.x + mark.w,
-	      mark.y + mark.h, 0, 0, 0, 1);
+	      mark.y + mark.h, 0, 0, 0, 1, curpen);
   FreePix ();
   CreatePix ();
   Display_Mol ();
@@ -1717,21 +1879,21 @@ add_r_bracket ()
     return;
   gauge = 10 + (boxgap * mark.h / 600 * mark.w / 600);
   add_struct (mark.x, mark.y + gauge, mark.x, mark.y + mark.h - gauge, 0, 0,
-	      0, 1);
+	      0, 1, curpen);
   add_struct (mark.x + mark.w, mark.y + gauge, mark.x + mark.w,
-	      mark.y + mark.h - gauge, 0, 0, 0, 1);
+	      mark.y + mark.h - gauge, 0, 0, 0, 1, curpen);
   add_spline (mark.x, mark.y + gauge, mark.x, mark.y + gauge / 2,
-	      mark.x + gauge / 2, mark.y, mark.x + gauge, mark.y, 0, 0);
+	      mark.x + gauge / 2, mark.y, mark.x + gauge, mark.y, 0, 0,curpen);
   add_spline (mark.x + mark.w - gauge, mark.y, mark.x + mark.w - gauge / 2,
 	      mark.y, mark.x + mark.w, mark.y + gauge / 2, mark.x + mark.w,
-	      mark.y + gauge, 0, 0);
+	      mark.y + gauge, 0, 0,curpen);
   add_spline (mark.x, mark.y + mark.h - gauge, mark.x,
 	      mark.y + mark.h - gauge / 2, mark.x + gauge / 2,
-	      mark.y + mark.h, mark.x + gauge, mark.y + mark.h, 0, 0);
+	      mark.y + mark.h, mark.x + gauge, mark.y + mark.h, 0, 0,curpen);
   add_spline (mark.x + mark.w - gauge, mark.y + mark.h,
 	      mark.x + mark.w - gauge / 2, mark.y + mark.h, mark.x + mark.w,
 	      mark.y + mark.h - gauge / 2, mark.x + mark.w,
-	      mark.y + mark.h - gauge, 0, 0);
+	      mark.y + mark.h - gauge, 0, 0,curpen);
   FreePix ();
   CreatePix ();
   Display_Mol ();
@@ -1746,17 +1908,17 @@ add_brace ()
     return;
   gauge = 10 + (boxgap * mark.h / 600 * mark.w / 600);
   add_spline (mark.x + gauge, mark.y, mark.x - gauge, mark.y, mark.x + gauge,
-	      mark.y + mark.h / 2, mark.x - gauge, mark.y + mark.h / 2, 0, 0);
+	      mark.y + mark.h / 2, mark.x - gauge, mark.y + mark.h / 2, 0, 0,curpen);
   add_spline (mark.x - gauge, mark.y + mark.h / 2, mark.x + gauge,
 	      mark.y + mark.h / 2, mark.x - gauge, mark.y + mark.h,
-	      mark.x + gauge, mark.y + mark.h, 0, 0);
+	      mark.x + gauge, mark.y + mark.h, 0, 0,curpen);
   add_spline (mark.x + mark.w - gauge, mark.y, mark.x + mark.w + gauge,
 	      mark.y, mark.x + mark.w - gauge, mark.y + mark.h / 2,
-	      mark.x + mark.w + gauge, mark.y + mark.h / 2, 0, 0);
+	      mark.x + mark.w + gauge, mark.y + mark.h / 2, 0, 0,curpen);
   add_spline (mark.x + mark.w + gauge, mark.y + mark.h / 2,
 	      mark.x + mark.w - gauge, mark.y + mark.h / 2,
 	      mark.x + mark.w + gauge, mark.y + mark.h,
-	      mark.x + mark.w - gauge, mark.y + mark.h, 0, 0);
+	      mark.x + mark.w - gauge, mark.y + mark.h, 0, 0,curpen);
   FreePix ();
   CreatePix ();
   Display_Mol ();
@@ -1770,12 +1932,12 @@ add_box1 ()
   if (importflag != 0)
     return;
 
-  add_struct (mark.x, mark.y, mark.x + mark.w, mark.y, 0, 0, 0, 1);
-  add_struct (mark.x, mark.y, mark.x, mark.y + mark.h, 0, 0, 0, 1);
+  add_struct (mark.x, mark.y, mark.x + mark.w, mark.y, 0, 0, 0, 1, curpen);
+  add_struct (mark.x, mark.y, mark.x, mark.y + mark.h, 0, 0, 0, 1, curpen);
   add_struct (mark.x, mark.y + mark.h, mark.x + mark.w, mark.y + mark.h, 0, 0,
-	      0, 1);
+	      0, 1, curpen);
   add_struct (mark.x + mark.w, mark.y, mark.x + mark.w, mark.y + mark.h, 0, 0,
-	      0, 1);
+	      0, 1, curpen);
   FreePix ();
   CreatePix ();
   Display_Mol ();
@@ -1788,16 +1950,16 @@ add_box2 ()
   if (importflag != 0)
     return;
 
-  add_struct (mark.x, mark.y, mark.x + mark.w, mark.y, 0, 0, 0, 1);
-  add_struct (mark.x, mark.y, mark.x, mark.y + mark.h, 0, 0, 0, 1);
+  add_struct (mark.x, mark.y, mark.x + mark.w, mark.y, 0, 0, 0, 1, curpen);
+  add_struct (mark.x, mark.y, mark.x, mark.y + mark.h, 0, 0, 0, 1, curpen);
   add_struct (mark.x, mark.y + mark.h, mark.x + mark.w, mark.y + mark.h, 0, 0,
-	      0, 1);
+	      0, 1, curpen);
   add_struct (mark.x + mark.w, mark.y, mark.x + mark.w, mark.y + mark.h, 0, 0,
-	      0, 1);
+	      0, 1, curpen);
   add_struct (mark.x + mark.w + 2, mark.y + 3, mark.x + mark.w + 2,
-	      mark.y + mark.h + 3, 0, 0, 0, 1);
+	      mark.y + mark.h + 3, 0, 0, 0, 1, curpen);
   add_struct (mark.x + mark.w + 3, mark.y + mark.h + 2, mark.x + 3,
-	      mark.y + mark.h + 2, 0, 0, 0, 1);
+	      mark.y + mark.h + 2, 0, 0, 0, 1, curpen);
   FreePix ();
   CreatePix ();
   Display_Mol ();
@@ -1810,16 +1972,16 @@ add_box3 ()
   if (importflag != 0)
     return;
 
-  add_struct (mark.x, mark.y, mark.x + mark.w, mark.y, 0, 0, 0, 1);
-  add_struct (mark.x, mark.y, mark.x, mark.y + mark.h, 0, 0, 0, 1);
+  add_struct (mark.x, mark.y, mark.x + mark.w, mark.y, 0, 0, 0, 1, curpen);
+  add_struct (mark.x, mark.y, mark.x, mark.y + mark.h, 0, 0, 0, 1, curpen);
   add_struct (mark.x, mark.y + mark.h, mark.x + mark.w, mark.y + mark.h, 0, 0,
-	      0, 1);
+	      0, 1, curpen);
   add_struct (mark.x + mark.w, mark.y, mark.x + mark.w, mark.y + mark.h, 0, 0,
-	      0, 1);
+	      0, 1, curpen);
   add_struct (mark.x + mark.w + 5, mark.y + 8, mark.x + mark.w + 5,
-	      mark.y + mark.h + 8, 10, 0, 0, 1);
+	      mark.y + mark.h + 8, 10, 0, 0, 1, curpen);
   add_struct (mark.x + mark.w + 9, mark.y + mark.h + 5, mark.x + 9,
-	      mark.y + mark.h + 5, 10, 0, 0, 1);
+	      mark.y + mark.h + 5, 10, 0, 0, 1, curpen);
   FreePix ();
   CreatePix ();
   Display_Mol ();
@@ -1833,25 +1995,68 @@ add_box4 ()
     return;
 
   add_struct (mark.x + boxgap, mark.y, mark.x + mark.w - boxgap, mark.y, 0, 0,
-	      0, 1);
+	      0, 1, curpen);
   add_struct (mark.x, mark.y + boxgap, mark.x, mark.y + mark.h - boxgap, 0, 0,
-	      0, 1);
+	      0, 1, curpen);
   add_struct (mark.x + boxgap, mark.y + mark.h, mark.x + mark.w - boxgap,
-	      mark.y + mark.h, 0, 0, 0, 1);
+	      mark.y + mark.h, 0, 0, 0, 1, curpen);
   add_struct (mark.x + mark.w, mark.y + boxgap, mark.x + mark.w,
-	      mark.y + mark.h - boxgap, 0, 0, 0, 1);
+	      mark.y + mark.h - boxgap, 0, 0, 0, 1, curpen);
   add_spline (mark.x, mark.y + boxgap, mark.x, mark.y, mark.x, mark.y,
-	      mark.x + boxgap, mark.y, 0, 0);
+	      mark.x + boxgap, mark.y, 0, 0,curpen);
   add_spline (mark.x + mark.w - boxgap, mark.y, mark.x + mark.w, mark.y,
 	      mark.x + mark.w, mark.y, mark.x + mark.w, mark.y + boxgap, 0,
-	      0);
+	      0,curpen);
   add_spline (mark.x, mark.y + mark.h - boxgap, mark.x, mark.y + mark.h,
 	      mark.x, mark.y + mark.h, mark.x + boxgap, mark.y + mark.h, 0,
-	      0);
+	      0,curpen);
   add_spline (mark.x + mark.w - boxgap, mark.y + mark.h, mark.x + mark.w,
 	      mark.y + mark.h, mark.x + mark.w, mark.y + mark.h,
-	      mark.x + mark.w, mark.y + mark.h - boxgap, 0, 0);
+	      mark.x + mark.w, mark.y + mark.h - boxgap, 0, 0,curpen);
   FreePix ();
   CreatePix ();
   Display_Mol ();
 }
+
+
+int
+has_label(int x, int y)
+/* checks if a given coordinate (i.e. bond-limit) has a label/atom on it */
+{
+  int d;
+  struct dc *hp_atom;  
+  hp_atom = dac_root.next;
+  for (d = 0; d < hp->nc; d++)
+    {
+      if ( abs(hp_atom->x-x)<5  && abs(hp_atom->y-y)<5 )
+ 	return hp_atom->size;
+      hp_atom = hp_atom->next;
+    }
+  return -1;
+}
+ 
+int
+endpoint_connected(int x, int y, int tx, int ty)
+/* checks if the endpoint of a bond has another bond on it */
+{
+  int d;
+  struct data *hp_bond;
+  hp_bond = da_root.next;
+  for (d = 0; d < hp->n; d++) {
+    if (hp_bond->x == tx && hp_bond->y == ty) {
+      /* this bond starts at our endpoint */
+      if (hp_bond->tx != x && hp_bond->ty != y)
+        /* and its not 'our' bond, so return true */
+        return 0;
+    }
+    if (hp_bond->tx == tx && hp_bond->ty == ty) {
+      /* same as above, it ends at our endpoint this time though */
+      if (hp_bond->x != x && hp_bond->y != y)
+        return 0;
+    }
+    hp_bond = hp_bond->next;
+  }
+  return 1;
+}
+
+
