@@ -5,25 +5,24 @@
 #include <string.h>
 
 #include "ct1.h"
-
 void
 Add_atom (int event_x, int event_y)
 /* reads the string from the text entry box and inserts it at the current
    cursor position in the current justification */
 {
   struct xy_co *coord;
+  struct dc *hpc;
   gchar *name;
-
   int x, y;
   int i;
 
-  name = gtk_entry_get_text (GTK_ENTRY (textbox));
+  name = gtk_entry_get_text (GTK_ENTRY (textbox)); 
   if (strlen (name))
     {
       x = round_coord (event_x, size_factor);
       y = round_coord (event_y, size_factor);
       coord = position (x, y, x, y, No_angle);
-      for (i = 0; i < strlen (name); i++)
+      for (i = 0; i < (int)strlen (name); i++)
 	{
 	  if (name[i] == ' ')
 	    name[i] = '\\';
@@ -34,10 +33,20 @@ Add_atom (int event_x, int event_y)
 	    }
 	}
       del_char (select_char (event_x, event_y, size_factor));
-      add_char (coord->x, coord->y, name, text_direct, 0);
+      add_char (coord->x, coord->y, name, text_direct, 0, curpen, serif_flag, curfontsize);
       modify = True;
       Display_Mol ();
-    }
+    } else {
+    hpc = select_char (event_x, event_y, size_factor);
+	if (hpc) {
+		hpc->direct=text_direct;
+		hpc->color=curpen;
+		hpc->font=serif_flag;
+		hpc->size=curfontsize;
+     		modify = True;
+      		Display_Mol ();
+	}
+    } 
 }
 
 void
@@ -86,6 +95,15 @@ Set_vector (int event_x, int event_y, int curbond)
       gdk_draw_rectangle (picture, drawing_area->style->black_gc,
 			  TRUE, coord->x * size_factor - 2,
 			  coord->y * size_factor - 2, 4, 4);
+#if 0
+/* draw a second ring of markers farther out */
+      gdk_draw_rectangle (picture, drawing_area->style->black_gc,
+			  TRUE, (2*coord->x-hp->x)* size_factor - 2,
+			  (2*coord->y-hp->y)*size_factor - 2, 4, 4);
+      gdk_draw_rectangle (picture, drawing_area->style->black_gc,
+			  TRUE, (3*coord->x-2*hp->x)* size_factor - 2,
+			  (3*coord->y-2*hp->y)*size_factor - 2, 4, 4);
+#endif
     }
   CopyPlane ();
 }
@@ -96,7 +114,7 @@ Add_vector (int event_x, int event_y, int cbond)
 {
   if ((hp->x - hp->tx) || (hp->y - hp->ty))
     {
-      add_struct (hp->x, hp->y, hp->tx, hp->ty, cbond, 0, 0, 0);
+      add_struct (hp->x, hp->y, hp->tx, hp->ty, cbond, 0, 0, 0,curpen);
   modify = True;
     }
   Display_Mol ();
@@ -114,7 +132,7 @@ Add_ring (int event_x, int event_y, int draw_angle, int curbond, int ringtype)
   double o_angle;		/* outside angle of the polygon */
   double i_angle;		/* inside angle of the polygon */
   double a;			/* temp angle used for aromates */
-  double m_x, m_y;		/* center point */
+  double mx, my;		/* center point */
   int aromatic = 0;		/* logical: draw aromate? */
   float percent = 0.7;		/* percentage of ring radius/polygon radius */
   int x0, y0, x1, y1;		/* start and end points of the drawn side 
@@ -155,8 +173,8 @@ Add_ring (int event_x, int event_y, int draw_angle, int curbond, int ringtype)
   i_angle = M_PI - o_angle;
 
   length =
-    sqrt ((hp->tx - hp->x) * (hp->tx - hp->x) +
-	  (hp->ty - hp->y) * (hp->ty - hp->y));
+    sqrt ((double)((hp->tx - hp->x) * (hp->tx - hp->x) +
+	  (hp->ty - hp->y) * (hp->ty - hp->y)));
 
   /* 'radius' is one cathete of a triangle between one half of a side of the
    * polygon and the line between one point and the center */
@@ -214,27 +232,27 @@ Add_ring (int event_x, int event_y, int draw_angle, int curbond, int ringtype)
        * and the line from one of those points to the center 
        * 'angle' is used to compensate for arbitrary bonds drawn 
        * by the user */
-      m_x = hp->x + cos (angle - (0.5 * i_angle)) * radius;
-      m_y = hp->y - sin (angle - (0.5 * i_angle)) * radius;
-/*	fprintf(stderr, "m_x: %f, m_y: %f, angle: %f\n", m_x, m_y, angle);*/
+      mx = hp->x + cos (angle - (0.5 * i_angle)) * radius;
+      my = hp->y - sin (angle - (0.5 * i_angle)) * radius;
+/*	fprintf(stderr, "mx: %f, my: %f, angle: %f\n", mx, my, angle);*/
 
       /* draw the polygon */
       for (i = 0; i < type; i++)
 	{
 	  x0 =
-	    (int) rint (m_x +
+	    (int) rint (mx +
 			radius * sin (i * o_angle +
 				      (3 * M_PI / type - angle)));
 	  y0 =
-	    (int) rint (m_y -
+	    (int) rint (my -
 			radius * cos (i * o_angle +
 				      (3 * M_PI / type - angle)));
 	  x1 =
-	    (int) rint (m_x +
+	    (int) rint (mx +
 			radius * sin ((i + 1) * o_angle +
 				      (3 * M_PI / type - angle)));
 	  y1 =
-	    (int) rint (m_y -
+	    (int) rint (my -
 			radius * cos ((i + 1) * o_angle +
 				      (3 * M_PI / type - angle)));
 	  /* if conjugated double bonds are selected, 
@@ -251,20 +269,11 @@ Add_ring (int event_x, int event_y, int draw_angle, int curbond, int ringtype)
 		{
 		  a = (acos ((x1 - x0) / length));
 		}
-	      /* make sure the double bond is painted inside the
-	       * polygon */
-	      if ((a > 0 && a <= M_PI / 2) || (a > M_PI && a <= 3 * M_PI / 2))
-		{
-		  add_struct (x0, y0, x1, y1, 2, 1, 1, 0);
-		}
-	      else
-		{
-		  add_struct (x0, y0, x1, y1, 1, 1, 1, 0);
-		}
+	      add_struct (x0, y0, x1, y1, 2, 1, 1, 0, curpen);
 	    }
 	  else
 	    {
-	      add_struct (x0, y0, x1, y1, curbond, 1, 1, 0);
+	      add_struct (x0, y0, x1, y1, curbond, 1, 1, 0, curpen);
 	    }
 	}
       /* draw a benzene ring into the polygon if curbond is ring */
@@ -275,9 +284,9 @@ Add_ring (int event_x, int event_y, int draw_angle, int curbond, int ringtype)
 	  y0 = (hp->ty + hp->y) / 2;
 	  /* get the point on the line from the center 70% to the
 	   * point above */
-	  x0 = ((x0 + percent * m_x) / (1.0 + percent));
-	  y0 = ((y0 + percent * m_y) / (1.0 + percent));
-	  add_struct (m_x, m_y, x0, y0, 11, 1, 1, 0);
+	  x0 = (int)((x0 + percent * mx) / (1.0 + percent));
+	  y0 = (int)((y0 + percent * my) / (1.0 + percent));
+	  add_struct ((int)mx, (int)my, x0, y0, 11, 1, 1, 0,curpen);
 	}
 
     }
@@ -301,7 +310,7 @@ Put_vector (int event_x, int event_y)
   if (x != hp->x || y != hp->y)
     {
       if (hp->x > 0)
-	gdk_draw_line (picture, drawing_area->style->white_gc,
+	gdk_draw_line (picture, background_gc,
 		       hp->x * size_factor, hp->y * size_factor,
 		       hp->tx * size_factor, hp->ty * size_factor);
       hp->tx = x;
@@ -366,6 +375,8 @@ Add_double (int event_x, int event_y)
    15 - right partial double
    16 - dashed bond
    17 - curved half arrow (shortcut)
+   18 - uniform triple
+   19 - quadruple
  */
 {
   struct data *hdc;
@@ -376,10 +387,11 @@ Add_double (int event_x, int event_y)
     {
       dx = hdc->x - hdc->tx;
       dy = hdc->y - hdc->ty;
-      if (hdc->bond >= 16)
+      if (hdc->bond >= 19)
 	hdc->bond = 0;
       else
 	++hdc->bond;
+      if (hdc->bond==17) hdc->bond++; /* no bond can become a curved arrow */	
       modify = True;
     }
   Display_Mol ();
@@ -394,13 +406,14 @@ Set_bondtype (int event_x, int event_y, int curbond)
   struct xy_co *coord;
   int d;
 
-  if (curbond > 15)
+  if (curbond == 17 && curbond > 19)
     return;			/* cannot change anything to 3-point spline */
 
   hdc = select_vector (event_x, event_y, size_factor);
   if (hdc)
     {
       hdc->bond = curbond;
+      hdc->color = curpen; 
       modify = True;
     }
   else
@@ -440,6 +453,7 @@ Set_bondtype (int event_x, int event_y, int curbond)
 		    default:
 		      break;
 		    }
+		    hsp->color = curpen;
 		  modify = True;
 		}
 	      hsp = hsp->next;
@@ -457,23 +471,71 @@ Set_start_rec (int event_x, int event_y)
   hp->y = event_y;
   hp->tx = event_x;
   hp->ty = event_y;
+  hp->startx = event_x;
+  hp->starty= event_y;
 }
 
 void
 Put_rec (int event_x, int event_y)
 /* draws the rubberband rectangle in mark mode */
 {
-  gdk_draw_rectangle (picture, drawing_area->style->white_gc,
-		      FALSE, hp->x, hp->y, hp->tx - hp->x, hp->ty - hp->y);
+   gdk_draw_rectangle (picture, background_gc,
+           FALSE, hp->x, hp->y, hp->tx - hp->x, hp->ty - hp->y);
 
-  hp->tx = event_x;
-  hp->ty = event_y;
-  if (hp->tx < hp->x)
-    hp->tx = hp->x;
-  if (hp->ty < hp->y)
-    hp->ty = hp->y;
+  if (event_x > hp->x) /* motion right */
+    {
+      if (event_x >= hp->startx)  /* right of startpoint */
+    {
+      hp->tx = event_x;
+      hp->x = hp->startx;  /* in case of too fast motion */
+    }
+      else
+    {
+      hp->x = event_x;  /* left of startpoint */
+      hp->tx = hp->startx;
+    }
+    }
+   else  /* motion left */
+    {
+      if (event_x > hp->startx)    /* right of startpoint */
+    {
+      hp->tx = event_x;
+      hp->x = hp->startx;
+    }
+      else
+    {
+      hp->x = event_x;   /* left of startpoint */
+      hp->tx = hp->startx;
+    }
+     }
+   if (event_y > hp->y)   /* motion down */
+    {
+      if (event_y > hp->starty)   /* below startpoint */
+    {
+      hp->ty = event_y;
+      hp->y = hp->starty;
+    }
+      else                    /* above startpoint */
+    {
+      hp->y = event_y;
+      hp->ty = hp->starty;
+    }
+    }
+   else        /* upward motion  */
+    {
+      if (event_y > hp->starty)     /* below startpoint */
+    {
+      hp->ty = event_y;
+      hp->y = hp->starty;
+    }
+      else                    /* above startpoint */
+    {
+      hp->y = event_y;
+      hp->ty = hp->starty;
+    }
+     }
   gdk_draw_rectangle (picture, drawing_area->style->black_gc,
-		      FALSE, hp->x, hp->y, hp->tx - hp->x, hp->ty - hp->y);
+              FALSE, hp->x, hp->y, hp->tx - hp->x, hp->ty - hp->y);
 
   CopyPlane ();
 }
@@ -519,9 +581,11 @@ Mark_rec (int event_x, int event_y, int control_pressed)
   undo_snapshot ();
 #endif
   CopyPlane ();
-  if (hp->tx == hp->x && hp->ty == hp->y)
+  if (hp->tx == hp->x && hp->ty == hp->y && !control_pressed)
     {
       mark.flag = False;
+      Mark_atom(hp->x,hp->y);
+      return;
     }
   else
     {
@@ -616,10 +680,20 @@ Mark_rec (int event_x, int event_y, int control_pressed)
 	  hp_spline = hp_spline->next;
 	}
     }
-
+	if (addflag ==1) /* check if marker is within region */
+	  if (check_pos_rec
+	      (refx, refy, mark.x, mark.y, mark.x + mark.w,
+	       mark.y + mark.h)) refmark=1;
+	       
 /*reset mark.flag, if we did not find anything to mark */
+/*fprintf(stderr,"marks %d\n",marks);*/
   if (marks == 0)
     mark.flag = False;
+/*  if (marks == 1) {
+  	Unmark_all();
+  	Mark_atom(hp->x,hp->y);
+  	}
+*/
 }
 
 void
@@ -628,6 +702,7 @@ Unmark_all ()
 {
   struct data *hp_bond;
   struct dc *hp_atom;
+  struct spline *hp_spline;
   int hn;
   int d;
 
@@ -652,25 +727,49 @@ Unmark_all ()
       hp_bond->tmarked = 0;
       hp_bond = hp_bond->next;
     }
+      hp_spline = sp_root.next;
+      hn = hp->nsp;
+      for (d = 0; d < hn; d++)
+	{
+	hp_spline->marked = 0;
+	hp_spline = hp_spline->next;
+	}
+    if (addflag==1) refmark=0;
 }
 
 
 void
-Mark_atom (int event_x, int event_y)
+Mark_atom (int t_x, int t_y)
 /* marks a single position (for addition of fragments) */
 {
-  if (hp->tx < hp->x)
-    hp->tx = hp->x;
-  if (hp->ty < hp->y)
-    hp->ty = hp->y;
+  struct data *hdc;
+  int d, dt,hn,n;
 
-  mark.x = round_coord (hp->x, size_factor);
-  mark.y = round_coord (hp->y, size_factor);
-  mark.w = round_coord (hp->tx - hp->x, size_factor);
-  mark.h = round_coord (hp->ty - hp->y, size_factor);
-  mark.flag = True;
+  refx=-1;
+  refy=-1;
+	t_x=round_coord(t_x,size_factor);
+	t_y=round_coord(t_y,size_factor);
+  hdc = da_root.next;
+  hn = hp->n;
+  for (n = 0; n < hn; n++)
+    {
+  d = calc_vector(abs(hdc->x-t_x),abs(hdc->y-t_y));
+  dt = calc_vector(abs(hdc->tx-t_x),abs(hdc->ty-t_y));
+	if (d <10*size_factor) {
+	  refx = hdc->x;
+  	  refy = hdc->y;
+	  }else if (dt <10*size_factor) {
+	  refx = hdc->tx;
+  	  refy = hdc->ty;
+	}
+    	hdc=hdc->next;
+    }
+  if (refx<0 || refy<0)  {
+/*  fprintf(stderr,"no match found for %d %d\n",t_x,t_y);*/
+  addflag=0;
+  return;
+  }
   addflag = 1;
-  mark.flag = False;
   Display_Mol ();
 }
 
@@ -696,7 +795,7 @@ Set_pmove (int event_x, int event_y)
 
   int x, y;
   int d, dtmp;
-
+  draw_ok=1;
   x = round_coord (event_x, size_factor);
   y = round_coord (event_y, size_factor);
   coord = position (x, y, x, y, No_angle);
@@ -707,7 +806,7 @@ Set_pmove (int event_x, int event_y)
   get_center_marked (&xcent, &ycent);
 
   dtmp = 0;
-  if (tmpx)
+/*  if (tmpx)
     {
       free (tmpx);
       tmpx = NULL;
@@ -716,9 +815,9 @@ Set_pmove (int event_x, int event_y)
     {
       free (tmpy);
       tmpy = NULL;
-    }
-  tmpx = malloc ((2 * hp->n + hp->nc + 4 * hp->nsp) * sizeof (int));
-  tmpy = malloc ((2 * hp->n + hp->nc + 4 * hp->nsp) * sizeof (int));
+    }*/
+  tmpx = realloc (tmpx,(2 * hp->n + hp->nc + 4 * hp->nsp +1) * sizeof (int));
+  tmpy = realloc (tmpy,(2 * hp->n + hp->nc + 4 * hp->nsp +1) * sizeof (int));
   hp_bond = da_root.next;
   for (d = 0; d < hp->n; d++)
     {
@@ -763,6 +862,10 @@ Set_pmove (int event_x, int event_y)
 	}
       hp_spline = hp_spline->next;
     }
+    	if (addflag==1 && refmark==1){
+	  tmpx[dtmp] = refx - xcent;
+	  tmpy[dtmp++] = refy - ycent;
+    	}	
 }
 
 void
@@ -775,6 +878,7 @@ Put_pmove (int event_x, int event_y, int axis)
   y = round_coord (event_y, size_factor);
   if (importflag != 0)
     return;
+  if (!draw_ok) return;  
   if (mark.flag)
     {
       dx = x - hp->x;
@@ -798,7 +902,14 @@ Put_pmove (int event_x, int event_y, int axis)
 	  hp->y = y;
 	  Display_Mol ();
 	  modify = True;
-	}
+	} 
+#if 0
+	/* automatically switch to rubberband rectangle if nothing selected */
+	else {
+	Set_start_rec(event_x,event_y);
+    	drawmode = 4;
+    	}
+#endif
     }
 }
 
@@ -814,7 +925,7 @@ Put_protate (int event_x, int event_y, int control_pressed)
   static float ang = 0.;
   double rotsin = 0.;
   double rotcos = 0.;
-
+  if (!draw_ok) return;
   x = round_coord (event_x, size_factor);
   y = round_coord (event_y, size_factor);
   if (importflag != 0)
@@ -850,7 +961,7 @@ Put_protate (int event_x, int event_y, int control_pressed)
     }
   for (i = 0; i < 30; i++)
     {
-      if (abs ((angle + (angle * (float) i)) - abs(ang)) < 4.0 && control_pressed)
+      if (fabs ((angle + (angle * (float) i)) - fabs(ang)) < 4.0 && control_pressed)
 	{
 	  rotsin = sin ((angle + (angle * (float) i)) * M_PI / 180.);
 	  rotcos = cos ((angle + (angle * (float) i)) * M_PI / 180.);
@@ -862,7 +973,7 @@ Put_protate (int event_x, int event_y, int control_pressed)
 	  rotcos = cos (ang * M_PI / 180.);
 	}
     }
-  if (partial_rotate (xcent, ycent, rotsin, rotcos))
+  if (partial_rotate (/*xcent, ycent,*/ rotsin, rotcos))
     modify = True;
   hp->x = x;
   hp->y = y;
@@ -883,6 +994,7 @@ Set_pscale (int event_x, int event_y)
   int x, y;
   int d, dtmp;
 
+  draw_ok=1;
   x = round_coord (event_x, size_factor);
   y = round_coord (event_y, size_factor);
   coord = position (x, y, x, y, No_angle);
@@ -956,6 +1068,7 @@ Put_pscale (int event_x, int event_y, int aniso)
 {
   int x;
   int dx;
+  if (!draw_ok) return;
   x = round_coord (event_x, size_factor);
 
   if (mark.flag)
@@ -967,3 +1080,26 @@ Put_pscale (int event_x, int event_y, int aniso)
       Display_Mol ();
     }
 }
+
+void
+Add_number (int event_x, int event_y)
+/* insert an automatically incremented atom number at the current
+   cursor position*/
+{
+  struct xy_co *coord;
+  gchar name[10];
+  int x, y;
+
+  snprintf(name,9,"%d",++atnum); 
+  if (strlen (name))
+    {
+      x = round_coord (event_x, size_factor);
+      y = round_coord (event_y, size_factor);
+      coord = position (x, y, x, y, No_angle);
+      del_char (select_char (event_x, event_y, size_factor));
+/*      add_char (coord->x, coord->y, name, text_direct, 0, curpen, serif_flag, curfontsize);*/
+      add_char (x, y, name, Middle_Text, 0, curpen, serif_flag, 0);
+      modify = True;
+      Display_Mol ();
+    }
+}
