@@ -38,6 +38,13 @@
 
 #include "config.h"
 
+#ifdef DISABLE_NLS
+#define _(Text) Text
+#else
+#include <libintl.h>
+#define _(Text) gettext(Text)
+#endif
+
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkbutton.h>
 #include <gtk/gtkentry.h>
@@ -61,6 +68,20 @@
 #include <gtk/gtkpixmap.h>
 #include "gtkintl.h"
 
+/* The Hurd doesn't define either PATH_MAX or MAXPATHLEN, so we put this * 
+ * in here, since the rest of the code in the file does require some     *
+ * fixed maximum                                                         *
+ */
+ 
+#ifndef MAXPATHLEN
+#  ifdef PATH_MAX
+#    define MAXPATHLEN PATH_MAX
+#  else
+#    define MAXPATHLEN 2048
+#  endif
+#endif
+ 
+
 #define DIR_LIST_WIDTH   180
 #define DIR_LIST_HEIGHT  180
 #define FILE_LIST_WIDTH  180
@@ -421,7 +442,7 @@ gtk_file_selection_match_char (gchar text, gchar * mask)
       maskc = g_strdup (mask + 1);	/* get the portion of mask inside [] */
 
       (*(strchr (maskc, ']'))) = 0;
-      s = strlen ((char *) maskc);
+      s =(gint) strlen ((char *) maskc);
 
       for (x = 0; x < s; x++)
 	{
@@ -468,7 +489,7 @@ gtk_file_selection_match_mask (gchar * text, gchar * mask)
 	  tc1 = 1;
 	  (*(strchr (mymask, ','))) = 0;
 	}
-      for (tc = 0; tc <= strlen (text); tc++)
+      for (tc = 0; tc <= (int)strlen (text); tc++)
 	{
 	  if (gtk_file_selection_match_mask (text + tc, mymask + 1))
 	    {
@@ -499,7 +520,7 @@ gtk_file_selection_match_mask (gchar * text, gchar * mask)
 	  tc = 1;
 	  (*(strchr (mask2, ','))) = 0;
 	}
-      for (tc1 = 0; tc1 <= strlen (text); tc1++)
+      for (tc1 = 0; tc1 <= (int)strlen (text); tc1++)
 	{
 	  if (gtk_file_selection_match_mask (text + tc1, mask2 + 1))
 	    {
@@ -524,7 +545,7 @@ gtk_file_selection_match_mask (gchar * text, gchar * mask)
 	  g_free (mymask);
 	  return 0;
 	}
-      for (tc = 0; tc <= strlen (text); tc++)
+      for (tc = 0; tc <= (int)strlen (text); tc++)
 	{
 	  if (gtk_file_selection_match_mask (text + tc, mask3 + 1))
 	    {
@@ -555,8 +576,8 @@ GtkType gtk_file_selection_get_type (void)
     {
       static const GtkTypeInfo filesel_info = {
 	"GtkFileSelection",
-	sizeof (GtkFileSelection),
-	sizeof (GtkFileSelectionClass),
+	(guint)sizeof (GtkFileSelection),
+	(guint)sizeof (GtkFileSelectionClass),
 	(GtkClassInitFunc) gtk_file_selection_class_init,
 	(GtkObjectInitFunc) gtk_file_selection_init,
 	/* reserved_1 */ NULL,
@@ -651,21 +672,21 @@ gtk_file_selection_init (GtkFileSelection * filesel)
 			       FALSE);
   gtk_box_pack_start (GTK_BOX (pulldown_hbox), filesel->history_combo, TRUE,
 		      TRUE, 0);
-  gtk_signal_connect (GTK_OBJECT
+  (void)gtk_signal_connect (GTK_OBJECT
 		      (((GtkCombo *) filesel->history_combo)->entry),
 		      "key-press-event",
 		      (GtkSignalFunc)
 		      gtk_file_selection_history_combo_callback,
 		      (gpointer) filesel);
 
-  gtk_signal_connect (GTK_OBJECT
+  (void)gtk_signal_connect (GTK_OBJECT
 		      (((GtkCombo *) filesel->history_combo)->list),
 		      "button-press-event",
 		      (GtkSignalFunc)
 		      gtk_file_selection_history_combo_list_callback,
 		      (gpointer) filesel);
 
-  gtk_signal_connect (GTK_OBJECT
+  (void)gtk_signal_connect (GTK_OBJECT
 		      (((GtkCombo *) filesel->history_combo)->list),
 		      "key-press-event",
 		      (GtkSignalFunc)
@@ -705,7 +726,7 @@ gtk_file_selection_init (GtkFileSelection * filesel)
   home_button = gtk_button_new ();
   gtk_container_add (GTK_CONTAINER (home_button), pixmapwidget);
   gtk_widget_show (home_button);
-  gtk_signal_connect (GTK_OBJECT (home_button), "clicked",
+  (void)gtk_signal_connect (GTK_OBJECT (home_button), "clicked",
 		      (GtkSignalFunc) gtk_file_selection_home_button,
 		      (gpointer) filesel);
   gtk_box_pack_start (GTK_BOX (hbox), home_button, TRUE, TRUE, 0);
@@ -720,7 +741,7 @@ gtk_file_selection_init (GtkFileSelection * filesel)
   prev_button = gtk_button_new ();
   gtk_container_add (GTK_CONTAINER (prev_button), pixmapwidget);
 
-  gtk_signal_connect (GTK_OBJECT (prev_button), "clicked",
+  (void)gtk_signal_connect (GTK_OBJECT (prev_button), "clicked",
 		      (GtkSignalFunc) gtk_file_selection_prev_button,
 		      (gpointer) filesel);
   gtk_widget_show (prev_button);
@@ -735,7 +756,7 @@ gtk_file_selection_init (GtkFileSelection * filesel)
   gdk_pixmap_unref (pixmap);
   up_button = gtk_button_new ();
   gtk_container_add (GTK_CONTAINER (up_button), pixmapwidget);
-  gtk_signal_connect (GTK_OBJECT (up_button), "clicked",
+  (void)gtk_signal_connect (GTK_OBJECT (up_button), "clicked",
 		      (GtkSignalFunc) gtk_file_selection_up_button,
 		      (gpointer) filesel);
   gtk_widget_show (up_button);
@@ -751,7 +772,7 @@ gtk_file_selection_init (GtkFileSelection * filesel)
   next_button = gtk_button_new ();
   gtk_container_add (GTK_CONTAINER (next_button), pixmapwidget);
   gtk_widget_show (next_button);
-  gtk_signal_connect (GTK_OBJECT (next_button), "clicked",
+  (void)gtk_signal_connect (GTK_OBJECT (next_button), "clicked",
 		      (GtkSignalFunc) gtk_file_selection_next_button,
 		      (gpointer) filesel);
   gtk_box_pack_start (GTK_BOX (hbox), next_button, TRUE, TRUE, 0);
@@ -766,7 +787,7 @@ gtk_file_selection_init (GtkFileSelection * filesel)
   refresh_button = gtk_button_new ();
   gtk_container_add (GTK_CONTAINER (refresh_button), pixmapwidget);
   gtk_widget_show (refresh_button);
-  gtk_signal_connect (GTK_OBJECT (refresh_button), "clicked",
+  (void)gtk_signal_connect (GTK_OBJECT (refresh_button), "clicked",
 		      (GtkSignalFunc) gtk_file_selection_refresh_button,
 		      (gpointer) filesel);
   gtk_box_pack_start (GTK_BOX (hbox), refresh_button, TRUE, TRUE, 0);
@@ -776,10 +797,10 @@ gtk_file_selection_init (GtkFileSelection * filesel)
   dir_title[1] = NULL;
   filesel->dir_list = gtk_clist_new_with_titles (1, (gchar **) dir_title);
   gtk_widget_set_usize (filesel->dir_list, DIR_LIST_WIDTH, DIR_LIST_HEIGHT);
-  gtk_signal_connect (GTK_OBJECT (filesel->dir_list), "select_row",
+  (void)gtk_signal_connect (GTK_OBJECT (filesel->dir_list), "select_row",
 		      (GtkSignalFunc) gtk_file_selection_dir_button,
 		      (gpointer) filesel);
-  gtk_signal_connect (GTK_OBJECT (filesel->dir_list), "unselect_row",
+  (void)gtk_signal_connect (GTK_OBJECT (filesel->dir_list), "unselect_row",
 		      (GtkSignalFunc) gtk_file_selection_undir_button,
 		      (gpointer) filesel);
   gtk_clist_column_titles_passive (GTK_CLIST (filesel->dir_list));
@@ -807,7 +828,7 @@ gtk_file_selection_init (GtkFileSelection * filesel)
 
   filesel->mask_entry = gtk_entry_new ();
   gtk_widget_show (filesel->mask_entry);
-  gtk_signal_connect (GTK_OBJECT (filesel->mask_entry), "activate",
+  (void)gtk_signal_connect (GTK_OBJECT (filesel->mask_entry), "activate",
 		      (GtkSignalFunc) gtk_file_selection_mask_entry_callback,
 		      (gpointer) filesel);
   gtk_box_pack_start (GTK_BOX (hbox), filesel->mask_entry, TRUE, TRUE, 0);
@@ -819,7 +840,7 @@ gtk_file_selection_init (GtkFileSelection * filesel)
   filesel->file_list = gtk_clist_new_with_titles (1, (gchar **) file_title);
   gtk_widget_set_usize (filesel->file_list, FILE_LIST_WIDTH,
 			FILE_LIST_HEIGHT);
-  gtk_signal_connect (GTK_OBJECT (filesel->file_list), "select_row",
+  (void)gtk_signal_connect (GTK_OBJECT (filesel->file_list), "select_row",
 		      (GtkSignalFunc) gtk_file_selection_file_button,
 		      (gpointer) filesel);
   gtk_clist_column_titles_passive (GTK_CLIST (filesel->file_list));
@@ -874,14 +895,14 @@ gtk_file_selection_init (GtkFileSelection * filesel)
   gtk_widget_show (label);
 
   filesel->selection_entry = gtk_entry_new ();
-  gtk_signal_connect (GTK_OBJECT (filesel->selection_entry),
+  (void)gtk_signal_connect (GTK_OBJECT (filesel->selection_entry),
 		      "key_press_event",
 		      (GtkSignalFunc) gtk_file_selection_key_press, filesel);
-  gtk_signal_connect_object (GTK_OBJECT (filesel->selection_entry),
+  (void)gtk_signal_connect_object (GTK_OBJECT (filesel->selection_entry),
 			     "focus_in_event",
 			     (GtkSignalFunc) gtk_widget_grab_default,
 			     GTK_OBJECT (filesel->ok_button));
-  gtk_signal_connect_object (GTK_OBJECT (filesel->selection_entry),
+  (void)gtk_signal_connect_object (GTK_OBJECT (filesel->selection_entry),
 			     "activate", (GtkSignalFunc) gtk_button_clicked,
 			     GTK_OBJECT (filesel->ok_button));
   gtk_box_pack_start (GTK_BOX (entry_vbox), filesel->selection_entry, TRUE,
@@ -892,7 +913,7 @@ gtk_file_selection_init (GtkFileSelection * filesel)
     {
       gchar err_buf[256];
 
-      sprintf (err_buf, _("Directory unreadable: %s"),
+      snprintf (err_buf,256, _("Directory unreadable: %s"),
 	       cmpl_strerror (cmpl_errno));
 
       gtk_label_set_text (GTK_LABEL (filesel->selection_text), err_buf);
@@ -926,7 +947,7 @@ gtk_file_selection_show_fileop_buttons (GtkFileSelection * filesel)
   if (!filesel->fileop_c_dir)
     {
       filesel->fileop_c_dir = gtk_button_new_with_label (_("Create Dir"));
-      gtk_signal_connect (GTK_OBJECT (filesel->fileop_c_dir), "clicked",
+      (void)gtk_signal_connect (GTK_OBJECT (filesel->fileop_c_dir), "clicked",
 			  (GtkSignalFunc) gtk_file_selection_create_dir,
 			  (gpointer) filesel);
       gtk_box_pack_start (GTK_BOX (filesel->button_area),
@@ -937,7 +958,7 @@ gtk_file_selection_show_fileop_buttons (GtkFileSelection * filesel)
   if (!filesel->fileop_del_file)
     {
       filesel->fileop_del_file = gtk_button_new_with_label (_("Delete File"));
-      gtk_signal_connect (GTK_OBJECT (filesel->fileop_del_file), "clicked",
+      (void)gtk_signal_connect (GTK_OBJECT (filesel->fileop_del_file), "clicked",
 			  (GtkSignalFunc) gtk_file_selection_delete_file,
 			  (gpointer) filesel);
       gtk_box_pack_start (GTK_BOX (filesel->button_area),
@@ -948,7 +969,7 @@ gtk_file_selection_show_fileop_buttons (GtkFileSelection * filesel)
   if (!filesel->fileop_ren_file)
     {
       filesel->fileop_ren_file = gtk_button_new_with_label (_("Rename File"));
-      gtk_signal_connect (GTK_OBJECT (filesel->fileop_ren_file), "clicked",
+      (void)gtk_signal_connect (GTK_OBJECT (filesel->fileop_ren_file), "clicked",
 			  (GtkSignalFunc) gtk_file_selection_rename_file,
 			  (gpointer) filesel);
       gtk_box_pack_start (GTK_BOX (filesel->button_area),
@@ -1006,9 +1027,9 @@ gtk_file_selection_set_filename (GtkFileSelection * filesel,
     }
   else
     {
-      gint len = MIN (MAXPATHLEN - 1, last_slash - filename + 1);
+      gint len = MIN ((int)MAXPATHLEN - 1, (int)(last_slash - filename + 1));
 
-      strncpy (buf, filename, len);
+      strncpy (buf, filename, (size_t)len);
       buf[len] = 0;
 
       name = last_slash + 1;
@@ -1056,7 +1077,7 @@ gtk_file_selection_complete (GtkFileSelection * filesel,
 
   if (strchr (pattern, '*') || strchr (pattern, '?'))
     {
-      for (x = strlen (pattern); x >= 0; x--)
+      for (x = (gint)strlen (pattern); x >= 0; x--)
 	{
 	  if (pattern[x] == '/')
 	    break;
@@ -1173,7 +1194,7 @@ gtk_file_selection_fileop_error (GtkFileSelection * fs, gchar * error_message)
 
   /* close button */
   button = gtk_button_new_with_label (_("Close"));
-  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
+  (void)gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
 			     (GtkSignalFunc) gtk_widget_destroy,
 			     (gpointer) dialog);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
@@ -1244,7 +1265,7 @@ gtk_file_selection_create_dir (GtkWidget * widget, gpointer data)
 
   /* main dialog */
   fs->fileop_dialog = dialog = gtk_dialog_new ();
-  gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
+  (void)gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
 		      (GtkSignalFunc) gtk_file_selection_fileop_destroy,
 		      (gpointer) fs);
   gtk_window_set_title (GTK_WINDOW (dialog), _("Create Directory"));
@@ -1274,7 +1295,7 @@ gtk_file_selection_create_dir (GtkWidget * widget, gpointer data)
 
   /* buttons */
   button = gtk_button_new_with_label (_("Create"));
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
+  (void)gtk_signal_connect (GTK_OBJECT (button), "clicked",
 		      (GtkSignalFunc) gtk_file_selection_create_dir_confirmed,
 		      (gpointer) fs);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
@@ -1283,7 +1304,7 @@ gtk_file_selection_create_dir (GtkWidget * widget, gpointer data)
   gtk_widget_show (button);
 
   button = gtk_button_new_with_label (_("Cancel"));
-  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
+  (void)gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
 			     (GtkSignalFunc) gtk_widget_destroy,
 			     (gpointer) dialog);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
@@ -1348,7 +1369,7 @@ gtk_file_selection_delete_file (GtkWidget * widget, gpointer data)
 
   /* main dialog */
   fs->fileop_dialog = dialog = gtk_dialog_new ();
-  gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
+  (void)gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
 		      (GtkSignalFunc) gtk_file_selection_fileop_destroy,
 		      (gpointer) fs);
   gtk_window_set_title (GTK_WINDOW (dialog), _("Delete File"));
@@ -1374,7 +1395,7 @@ gtk_file_selection_delete_file (GtkWidget * widget, gpointer data)
 
   /* buttons */
   button = gtk_button_new_with_label (_("Delete"));
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
+  (void)gtk_signal_connect (GTK_OBJECT (button), "clicked",
 		      (GtkSignalFunc)
 		      gtk_file_selection_delete_file_confirmed,
 		      (gpointer) fs);
@@ -1384,7 +1405,7 @@ gtk_file_selection_delete_file (GtkWidget * widget, gpointer data)
   gtk_widget_show (button);
 
   button = gtk_button_new_with_label (_("Cancel"));
-  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
+  (void)gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
 			     (GtkSignalFunc) gtk_widget_destroy,
 			     (gpointer) dialog);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
@@ -1453,7 +1474,7 @@ gtk_file_selection_rename_file (GtkWidget * widget, gpointer data)
 
   /* main dialog */
   fs->fileop_dialog = dialog = gtk_dialog_new ();
-  gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
+  (void)gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
 		      (GtkSignalFunc) gtk_file_selection_fileop_destroy,
 		      (gpointer) fs);
   gtk_window_set_title (GTK_WINDOW (dialog), _("Rename File"));
@@ -1485,11 +1506,11 @@ gtk_file_selection_rename_file (GtkWidget * widget, gpointer data)
 
   gtk_entry_set_text (GTK_ENTRY (fs->fileop_entry), fs->fileop_file);
   gtk_editable_select_region (GTK_EDITABLE (fs->fileop_entry),
-			      0, strlen (fs->fileop_file));
+			      0, (gint)strlen (fs->fileop_file));
 
   /* buttons */
   button = gtk_button_new_with_label (_("Rename"));
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
+  (void)gtk_signal_connect (GTK_OBJECT (button), "clicked",
 		      (GtkSignalFunc)
 		      gtk_file_selection_rename_file_confirmed,
 		      (gpointer) fs);
@@ -1499,7 +1520,7 @@ gtk_file_selection_rename_file (GtkWidget * widget, gpointer data)
   gtk_widget_show (button);
 
   button = gtk_button_new_with_label (_("Cancel"));
-  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
+  (void)gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
 			     (GtkSignalFunc) gtk_widget_destroy,
 			     (gpointer) dialog);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
@@ -1825,7 +1846,7 @@ gtk_file_selection_file_button (GtkWidget * widget,
   g_return_if_fail (fs != NULL);
   g_return_if_fail (GTK_IS_FILE_SELECTION (fs));
 
-  gtk_clist_get_text (GTK_CLIST (fs->file_list), row, 0, &temp);
+  (void)gtk_clist_get_text (GTK_CLIST (fs->file_list), row, 0, &temp);
   filename = g_strdup (temp);
 
   if (filename)
@@ -1864,7 +1885,7 @@ gtk_file_selection_dir_button (GtkWidget * widget,
   g_return_if_fail (fs != NULL);
   g_return_if_fail (GTK_IS_FILE_SELECTION (fs));
 
-  gtk_clist_get_text (GTK_CLIST (fs->dir_list), row, 0, &temp);
+  (void)gtk_clist_get_text (GTK_CLIST (fs->dir_list), row, 0, &temp);
   filename = g_strdup (temp);
 
   if (filename)
@@ -1929,7 +1950,7 @@ gtk_file_selection_undir_button (GtkWidget * widget,
   g_return_if_fail (fs != NULL);
   g_return_if_fail (GTK_IS_FILE_SELECTION (fs));
 
-  gtk_clist_get_text (GTK_CLIST (fs->dir_list), row, 0, &temp);
+  (void)gtk_clist_get_text (GTK_CLIST (fs->dir_list), row, 0, &temp);
   filename = g_strdup (temp);
 
   if (filename)
@@ -2109,15 +2130,15 @@ gtk_file_selection_populate (GtkFileSelection * fs,
 	}
       else
 	{
-	  selection_index = cmpl_last_valid_char (cmpl_state) -
-	    (strlen (rel_path) - strlen (rem_path));
+	  selection_index = (gint) ( cmpl_last_valid_char (cmpl_state) -
+	    ((gint)strlen (rel_path) - (gint)strlen (rem_path)) );
 	  if (fs->selection_entry)
 	    gtk_entry_set_text (GTK_ENTRY (fs->selection_entry), rem_path);
 	}
     }
   else
     {
-      if (fs->selection_entry)
+      if (fs->selection_entry){}
 	/* Here we need to take the old filename and keep it! */
 	/*gtk_entry_set_text (GTK_ENTRY (fs->selection_entry), ""); */
 	;
@@ -2150,7 +2171,7 @@ gtk_file_selection_abort (GtkFileSelection * fs)
 {
   gchar err_buf[256];
 
-  sprintf (err_buf, _("Directory unreadable: %s"),
+  snprintf (err_buf,256, _("Directory unreadable: %s"),
 	   cmpl_strerror (cmpl_errno));
 
   /*  BEEP gdk_beep();  */
@@ -2293,9 +2314,9 @@ tryagain:
   new_state->directory_sent_storage = NULL;
   new_state->last_valid_char = 0;
   new_state->updated_text = g_new (gchar, MAXPATHLEN);
-  new_state->updated_text_alloc = MAXPATHLEN;
+  new_state->updated_text_alloc = (gint)MAXPATHLEN;
   new_state->the_completion.text = g_new (gchar, MAXPATHLEN);
-  new_state->the_completion.text_alloc = MAXPATHLEN;
+  new_state->the_completion.text_alloc = (gint)MAXPATHLEN;
   new_state->user_dir_name_buffer = NULL;
   new_state->user_directories = NULL;
 
@@ -2587,7 +2608,7 @@ open_user_dir (gchar * text_to_complete, CompletionState * cmpl_state)
   if (first_slash)
     cmp_len = first_slash - text_to_complete - 1;
   else
-    cmp_len = strlen (text_to_complete + 1);
+    cmp_len = (gint)strlen (text_to_complete + 1);
 
   if (!cmp_len)
     {
@@ -2604,7 +2625,7 @@ open_user_dir (gchar * text_to_complete, CompletionState * cmpl_state)
       /* ~user/ */
       char *copy = g_new (char, cmp_len + 1);
       struct passwd *pwd;
-      strncpy (copy, text_to_complete + 1, cmp_len);
+      strncpy (copy, text_to_complete + 1, (size_t)cmp_len);
       copy[cmp_len] = 0;
       pwd = getpwnam (copy);
       g_free (copy);
@@ -2666,9 +2687,9 @@ open_new_dir (gchar * dir_name, struct stat *sbuf, gboolean stat_subdirs)
   sent->inode = sbuf->st_ino;
   sent->device = sbuf->st_dev;
 
-  path_buf_len = strlen (dir_name);
+  path_buf_len = (gint)strlen (dir_name);
 
-  if (path_buf_len > MAXPATHLEN)
+  if (path_buf_len > (gint)MAXPATHLEN)
     {
       cmpl_errno = CMPL_ERRNO_TOO_LONG;
       return NULL;
@@ -2686,11 +2707,11 @@ open_new_dir (gchar * dir_name, struct stat *sbuf, gboolean stat_subdirs)
 
   while ((dirent_ptr = readdir (directory)) != NULL)
     {
-      int entry_len = strlen (dirent_ptr->d_name);
+      int entry_len = (int)strlen (dirent_ptr->d_name);
       buffer_size += entry_len + 1;
       entry_count += 1;
 
-      if (path_buf_len + entry_len + 2 >= MAXPATHLEN)
+      if (path_buf_len + entry_len + 2 >= (int)MAXPATHLEN)
 	{
 	  cmpl_errno = CMPL_ERRNO_TOO_LONG;
 	  closedir (directory);
@@ -2739,7 +2760,7 @@ open_new_dir (gchar * dir_name, struct stat *sbuf, gboolean stat_subdirs)
 	sent->entries[i].is_dir = 1;
     }
 
-  qsort (sent->entries, sent->entry_count, sizeof (CompletionDirEntry),
+  qsort (sent->entries, (size_t)sent->entry_count, sizeof (CompletionDirEntry),
 	 compare_cmpl_dir);
 
   closedir (directory);
@@ -2761,6 +2782,7 @@ check_dir (gchar * dir_name, struct stat *result, gboolean * stat_subdirs)
     gboolean present;
     struct stat statbuf;
   }
+/*@ignore@ splint does not like the incomplete initializer, but it is OK */
   no_stat_dirs[] =
   {
     {
@@ -2775,9 +2797,9 @@ check_dir (gchar * dir_name, struct stat *result, gboolean * stat_subdirs)
       0}
     }
   };
-
-  static const gint n_no_stat_dirs =
-    sizeof (no_stat_dirs) / sizeof (no_stat_dirs[0]);
+/*@end@ splint should resume checking here */
+  static const gint n_no_stat_dirs = (int)
+   ( sizeof (no_stat_dirs) / sizeof (no_stat_dirs[0]));
   static gboolean initialized = FALSE;
 
   gint i;
@@ -2864,7 +2886,7 @@ attach_dir (CompletionDirSent * sent, gchar * dir_name,
 
   new_dir->sent = sent;
   new_dir->fullname = g_strdup (dir_name);
-  new_dir->fullname_len = strlen (dir_name);
+  new_dir->fullname_len = (gint)strlen (dir_name);
 
   return new_dir;
 }
@@ -2872,7 +2894,7 @@ attach_dir (CompletionDirSent * sent, gchar * dir_name,
 static gint
 correct_dir_fullname (CompletionDir * cmpl_dir)
 {
-  gint length = strlen (cmpl_dir->fullname);
+  gint length = (gint)strlen (cmpl_dir->fullname);
   struct stat sbuf;
 
   if (strcmp (cmpl_dir->fullname + length - 2, "/.") == 0)
@@ -2931,7 +2953,7 @@ correct_dir_fullname (CompletionDir * cmpl_dir)
 	return FALSE;
     }
 
-  cmpl_dir->fullname_len = strlen (cmpl_dir->fullname);
+  cmpl_dir->fullname_len = (gint)strlen (cmpl_dir->fullname);
 
   return TRUE;
 }
@@ -3041,7 +3063,7 @@ attempt_homedir_completion (gchar * text_to_complete,
 
   if (!cmpl_state->user_dir_name_buffer && !get_pwdb (cmpl_state))
     return NULL;
-  length = strlen (text_to_complete) - 1;
+  length = (gint) (strlen (text_to_complete) - 1);
 
   cmpl_state->user_completion_index += 1;
 
@@ -3065,9 +3087,9 @@ attempt_homedir_completion (gchar * text_to_complete,
       cmpl_state->the_completion.is_a_completion = 1;
       cmpl_state->the_completion.is_directory = 1;
 
-      append_completion_text ("~", cmpl_state);
+      (void)append_completion_text ("~", cmpl_state);
 
-      append_completion_text (cmpl_state->user_directories
+      (void)append_completion_text (cmpl_state->user_directories
 			      [cmpl_state->user_completion_index].login,
 			      cmpl_state);
 
@@ -3118,7 +3140,7 @@ append_completion_text (gchar * text, CompletionState * cmpl_state)
   if (!cmpl_state->the_completion.text)
     return NULL;
 
-  len = strlen (text) + strlen (cmpl_state->the_completion.text) + 1;
+  len = (int) (strlen (text) + strlen (cmpl_state->the_completion.text) + 1);
 
   if (cmpl_state->the_completion.text_alloc > len)
     {
@@ -3134,7 +3156,7 @@ append_completion_text (gchar * text, CompletionState * cmpl_state)
   cmpl_state->the_completion.text_alloc = i;
 
   cmpl_state->the_completion.text =
-    (gchar *) g_realloc (cmpl_state->the_completion.text, i);
+    (gchar *) g_realloc (cmpl_state->the_completion.text, (gulong)i);
 
   if (!cmpl_state->the_completion.text)
     return NULL;
@@ -3162,7 +3184,7 @@ find_completion_dir (gchar * text_to_complete,
       gint i;
       gchar *pat_buf = g_new (gchar, len + 1);
 
-      strncpy (pat_buf, *remaining_text, len);
+      strncpy (pat_buf, *remaining_text, (size_t)len);
       pat_buf[len] = 0;
 
       for (i = 0; i < dir->sent->entry_count; i += 1)
@@ -3225,13 +3247,13 @@ update_cmpl (PossibleCompletion * poss, CompletionState * cmpl_state)
   if (!poss || !cmpl_is_a_completion (poss))
     return;
 
-  cmpl_len = strlen (cmpl_this_completion (poss));
+  cmpl_len = (gint)strlen (cmpl_this_completion (poss));
 
   if (cmpl_state->updated_text_alloc < cmpl_len + 1)
     {
       cmpl_state->updated_text =
 	(gchar *) g_realloc (cmpl_state->updated_text,
-			     cmpl_state->updated_text_alloc);
+			     (gulong)cmpl_state->updated_text_alloc);
       cmpl_state->updated_text_alloc = 2 * cmpl_len;
     }
 
@@ -3296,12 +3318,12 @@ attempt_file_completion (CompletionState * cmpl_state)
       gint len = first_slash - dir->cmpl_text;
 
       pat_buf = g_new (gchar, len + 1);
-      strncpy (pat_buf, dir->cmpl_text, len);
+      strncpy (pat_buf, dir->cmpl_text, (size_t)len);
       pat_buf[len] = 0;
     }
   else
     {
-      gint len = strlen (dir->cmpl_text);
+      gint len = (gint)strlen (dir->cmpl_text);
 
       pat_buf = g_new (gchar, len + 2);
       strcpy (pat_buf, dir->cmpl_text);
@@ -3354,13 +3376,13 @@ attempt_file_completion (CompletionState * cmpl_state)
     {
       if (dir->cmpl_parent != NULL)
 	{
-	  append_completion_text (dir->fullname +
+	  (void)append_completion_text (dir->fullname +
 				  strlen (cmpl_state->completion_dir->
 					  fullname) + 1, cmpl_state);
-	  append_completion_text ("/", cmpl_state);
+	  (void)append_completion_text ("/", cmpl_state);
 	}
 
-      append_completion_text (dir->sent->entries[dir->cmpl_index].entry_name,
+      (void)append_completion_text (dir->sent->entries[dir->cmpl_index].entry_name,
 			      cmpl_state);
 
       cmpl_state->the_completion.is_a_completion =
@@ -3370,7 +3392,7 @@ attempt_file_completion (CompletionState * cmpl_state)
       cmpl_state->the_completion.is_directory =
 	dir->sent->entries[dir->cmpl_index].is_dir;
       if (dir->sent->entries[dir->cmpl_index].is_dir)
-	append_completion_text ("/", cmpl_state);
+	(void)append_completion_text ("/", cmpl_state);
 
       g_free (pat_buf);
       return &cmpl_state->the_completion;
@@ -3387,9 +3409,9 @@ get_pwdb (CompletionState * cmpl_state)
 
   if (cmpl_state->user_dir_name_buffer)
     return TRUE;
-  setpwent ();
+/*@i@*/  setpwent ();
 
-  while ((pwd_ptr = getpwent ()) != NULL)
+/*@i@*/  while ((pwd_ptr = getpwent ()) != NULL)
     {
       len += strlen (pwd_ptr->pw_name);
       len += strlen (pwd_ptr->pw_dir);
@@ -3397,7 +3419,7 @@ get_pwdb (CompletionState * cmpl_state)
       count += 1;
     }
 
-  setpwent ();
+/*@i@*/  setpwent ();
 
   cmpl_state->user_dir_name_buffer = g_new (gchar, len);
   cmpl_state->user_directories = g_new (CompletionUserDir, count);
@@ -3425,10 +3447,10 @@ get_pwdb (CompletionState * cmpl_state)
     }
 
   qsort (cmpl_state->user_directories,
-	 cmpl_state->user_directories_len,
+	 (size_t)cmpl_state->user_directories_len,
 	 sizeof (CompletionUserDir), compare_user_dir);
 
-  endpwent ();
+/*@i@*/  endpwent ();
 
   return TRUE;
 
@@ -3503,14 +3525,14 @@ main (int argc, char *argv[])
 //    gtk_file_selection_complete(GTK_FILE_SELECTION(filew),"bob");
 
 
-  gtk_signal_connect (GTK_OBJECT (filew), "destroy",
+  (void)gtk_signal_connect (GTK_OBJECT (filew), "destroy",
 		      (GtkSignalFunc) destroy, &filew);
   /* Connect the ok_button to file_ok_sel function */
-  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
+  (void)gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
 		      "clicked", (GtkSignalFunc) file_ok_sel, filew);
 
   /* Connect the cancel_button to destroy the widget */
-  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
+  (void)gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
 					 (filew)->cancel_button),
 			     "clicked", (GtkSignalFunc) gtk_widget_destroy,
 			     GTK_OBJECT (filew));
